package de.mbohlen.tools.jpresenter;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.ImageObserver;

import javax.swing.JPanel;

public class ImagePanel extends JPanel
{
    private Image theImage = null;
    int newWidth = 800, newHeight = 600;
    int imgW, imgH;

    public void setImage(Image i)
    {
        theImage = i;
        repaint();
    }

    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        int panelH = getHeight();
        int panelW = getWidth();
        if (theImage != null)
        {
            // scale the image to fit in the panel

            imgW = theImage.getWidth(this);
            imgH = theImage.getHeight(this);

            // because of the imageObserver interface, the calls above can
            // return immediately with a value of -1 which means that the image
            // is not fully loaded, so we just skip the rest of the paint method:

            if ((imgW == -1) || (imgH == -1))
                return;

            // image is taller than panel or image is wider than panel
            if (imgH > panelH)
            {
                newHeight = panelH;
                newWidth = (int) (panelH * imgW / imgH);
            }
            else if (imgW > panelW)
            {
                newWidth = panelW;
                newHeight = (int) (panelW * imgH / imgW);
            }
            // image is smaller than panel in width and length
            if (imgW < panelW && imgH < panelH)
            {
                newWidth = imgW;
                newHeight = imgH;
            }
            g.drawImage(
                theImage,
                (panelW - imgW) / 2,
                (panelH - imgH) / 2,
                newWidth,
                newHeight,
                this);
        }
    }

    public boolean imageUpdate(
        Image img,
        int infoflags,
        int x,
        int y,
        int width,
        int height)
    {
        // this is a callback method because calling getWidth or getHeight on an
        // image is likely to return immediately with a value of -1 if the image
        // is not fully loaded. The computer will call this method until it returns
        // true. We wait here until the width and the height of the image are known
        if ((infoflags & ImageObserver.WIDTH) != 0
            && (infoflags & ImageObserver.HEIGHT) != 0)
        {
            imgW = width;
            imgH = height;
            repaint();
            return false;
        }
        return true;
    }

}