package de.mbohlen.tools.jpresenter;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.sound.sampled.Clip;
import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;

public class MainFrame extends JFrame
{
    JPanel contentPane;
    JMenuBar jMenuBar1 = new JMenuBar();
    JMenu jMenuFile = new JMenu();
    JMenuItem jMenuFileExit = new JMenuItem();
    JMenu jMenuHelp = new JMenu();
    JMenuItem jMenuHelpAbout = new JMenuItem();
    JToolBar jToolBar = new JToolBar();
    JButton jFileOpenButton = new JButton();
    JButton jHelpButton = new JButton();
    ImageIcon image1;
    ImageIcon image2;
    ImageIcon image3;
    JLabel statusBar = new JLabel();
    BorderLayout borderLayout1 = new BorderLayout();
    JSplitPane jSplitPane1 = new JSplitPane();
    JList jPresItems = new JList();
    ImagePanel jImagePanel1 = new ImagePanel();
    TitledBorder titledBorder1;
    JMenuItem jMenuOpenPres = new JMenuItem();

    private Presentation presentation = null;
    private File thePresenterFile;
    private Clip theActiveClip = null;
    private PlaybackMonitor playbackMonitor = new PlaybackMonitor();

    //Construct the frame
    public MainFrame()
    {
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try
        {
            jbInit();
            myInit();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    private void myInit()
    {
        copyActionListeners(jMenuOpenPres, jFileOpenButton);
    }

    private void copyActionListeners(
        AbstractButton from,
        AbstractButton to)
    {
        ActionListener[] al = from.getActionListeners();
        for (int i = 0; i < al.length; i++)
        {
            to.addActionListener(al[i]);
        }
    }

    //Component initialization
    private void jbInit() throws Exception
    {
        image1 =
            new ImageIcon(
                de.mbohlen.tools.jpresenter.MainFrame.class.getResource(
                    "openFile.gif"));
        image2 =
            new ImageIcon(
                de.mbohlen.tools.jpresenter.MainFrame.class.getResource(
                    "closeFile.gif"));
        image3 =
            new ImageIcon(
                de.mbohlen.tools.jpresenter.MainFrame.class.getResource(
                    "help.gif"));
        //setIconImage(Toolkit.getDefaultToolkit().createImage(MainFrame.class.getResource("[Your Icon]")));
        contentPane = (JPanel) this.getContentPane();
        titledBorder1 = new TitledBorder("");
        contentPane.setLayout(borderLayout1);
        this.setSize(new Dimension(600, 400));
        this.setTitle("JPresenter");
        statusBar.setText(" ");
        jMenuFile.setText("File");
        jMenuFileExit.setText("Exit");
        jMenuFileExit.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuFileExit_actionPerformed(e);
            }
        });
        jMenuHelp.setText("Help");
        jMenuHelpAbout.setText("About");
        jMenuHelpAbout.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuHelpAbout_actionPerformed(e);
            }
        });
        jFileOpenButton.setIcon(image1);
        jFileOpenButton.setToolTipText("Open File");
        jHelpButton.setIcon(image3);
        jHelpButton.setToolTipText("Help");
        jPresItems.setBorder(BorderFactory.createEtchedBorder());
        jPresItems.setPreferredSize(new Dimension(200, 0));
        jPresItems.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        jPresItems
            .addListSelectionListener(
                new javax
                .swing
                .event
                .ListSelectionListener()
        {
            public void valueChanged(ListSelectionEvent e)
            {
                jPresItems_valueChanged(e);
            }
        });
        jMenuOpenPres.setText("Open Presentation...");
        jMenuOpenPres.setAccelerator(
            javax.swing.KeyStroke.getKeyStroke(
                79,
                java.awt.event.KeyEvent.CTRL_MASK,
                false));
        jMenuOpenPres.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuOpenPres_actionPerformed(e);
            }
        });
        jImagePanel1.setBorder(BorderFactory.createEtchedBorder());
        jSplitPane1.setBorder(null);
        jToolBar.add(jFileOpenButton);
        jToolBar.add(jHelpButton);
        jMenuFile.add(jMenuOpenPres);
        jMenuFile.add(jMenuFileExit);
        jMenuHelp.add(jMenuHelpAbout);
        jMenuBar1.add(jMenuFile);
        jMenuBar1.add(jMenuHelp);
        this.setJMenuBar(jMenuBar1);
        contentPane.add(jToolBar, BorderLayout.NORTH);
        contentPane.add(statusBar, BorderLayout.SOUTH);
        contentPane.add(jSplitPane1, BorderLayout.CENTER);
        jSplitPane1.add(jImagePanel1, JSplitPane.RIGHT);
        jSplitPane1.add(jPresItems, JSplitPane.LEFT);
    }
    //File | Exit action performed
    public void jMenuFileExit_actionPerformed(ActionEvent e)
    {
        System.exit(0);
    }
    //Help | About action performed
    public void jMenuHelpAbout_actionPerformed(ActionEvent e)
    {
        MainFrame_AboutBox dlg = new MainFrame_AboutBox(this);
        Dimension dlgSize = dlg.getPreferredSize();
        Dimension frmSize = getSize();
        Point loc = getLocation();
        dlg.setLocation(
            (frmSize.width - dlgSize.width) / 2 + loc.x,
            (frmSize.height - dlgSize.height) / 2 + loc.y);
        dlg.setModal(true);
        dlg.show();
    }
    //Overridden so we can exit when window is closed
    protected void processWindowEvent(WindowEvent e)
    {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_CLOSING)
        {
            jMenuFileExit_actionPerformed(null);
        }
    }

    void jMenuOpenPres_actionPerformed(ActionEvent e)
    {
        JFileChooser fc = new JFileChooser();
        fc.addChoosableFileFilter(
            new ExampleFileFilter("jpz", "Zipped JPresenter files"));
        fc.setMultiSelectionEnabled(false);
        fc.setDialogTitle("Open a JPresenter presentation");
        fc.setCurrentDirectory(new File("."));
        int userChoice = fc.showOpenDialog(this);

        if (userChoice == JFileChooser.APPROVE_OPTION)
        {
            thePresenterFile = fc.getSelectedFile();
            if (thePresenterFile.getName().toLowerCase().endsWith("jpz"))
            {
                presentation =
                    Presentation.unzipPresentation(thePresenterFile);

                if (presentation != null)
                {
                    this.setTitle(
                        "JPresenter - " + thePresenterFile.getPath());
                    statusBar.setText("Presentation loaded.");
                    jPresItems.setModel(
                        new PresentationListModel(presentation));
                    repaint();
                }
                else
                {
                    this.setTitle("JPresenter");
                    statusBar.setText("Presentation not loaded.");
                }

            }
            else
            {
                JOptionPane.showMessageDialog(
                    this,
                    "JPresenter cannot open files that are not presentations",
                    "Mild Error",
                    JOptionPane.ERROR_MESSAGE);
            }
        }

    }

    /**
     * The user has selected another slide from the list box.
     * Load image and sound from the zip file and perform them!
     * @param e unused parameter
     */
    synchronized void jPresItems_valueChanged(ListSelectionEvent e)
    {
        Slide selectedSlide = (Slide) jPresItems.getSelectedValue();

        try
        {
            Image image = selectedSlide.loadImage(thePresenterFile);
            jImagePanel1.setImage(image);
        }
        catch (Exception ex)
        {
            JOptionPane.showMessageDialog(
                this,
                ex.getMessage(),
                "Exception when loading the image of the selected slide",
                JOptionPane.ERROR_MESSAGE);
        }

        if (theActiveClip != null)
        {
            if (theActiveClip.isActive())
            {
                theActiveClip.stop();
                theActiveClip.close();
                theActiveClip = null;
            }
        }

        try
        {
            theActiveClip = selectedSlide.loadSound(thePresenterFile);
        }
        catch (Exception ex2)
        {
            JOptionPane.showMessageDialog(
                this,
                ex2.getMessage(),
                "Exception when loading the sound clip of the selected slide",
                JOptionPane.ERROR_MESSAGE);

        }

        if (theActiveClip != null)
        {
            playbackMonitor.start();
            theActiveClip.start();
        }
    }

    private synchronized void updateAudioStatus()
    {
        String s = String.valueOf(getSeconds());
        s = s.substring(0, s.indexOf('.') + 2);
        String d = String.valueOf(getDuration());
        d = d.substring(0, d.indexOf('.') + 2);
        statusBar.setText("Playing: " + s + " / " + d);
        statusBar.repaint();
    }

    private double getDuration()
    {
        double duration = 0.0;

        duration =
            theActiveClip.getBufferSize()
                / (theActiveClip.getFormat().getFrameSize()
                    * theActiveClip.getFormat().getFrameRate());
        return duration;
    }

    private double getSeconds()
    {
        double seconds =
            theActiveClip.getFramePosition()
                / theActiveClip.getFormat().getFrameRate();
        return seconds;
    }

    /**
     * Displays current sound and time elapsed.
     */
    public class PlaybackMonitor implements Runnable
    {

        Thread pbThread;

        public void start()
        {
            pbThread = new Thread(this);
            pbThread.setName("PlaybackMonitor");
            pbThread.start();
        }

        public void stop()
        {
            if (pbThread != null)
            {
                pbThread.interrupt();
            }
            pbThread = null;
        }

        public void run()
        {
            while (pbThread != null)
            {
                try
                {
                    pbThread.sleep(99);
                }
                catch (Exception e)
                {
                    break;
                }
                updateAudioStatus();
            }
            pbThread = null;
        }
    } // End PlaybackMonitor

}