package de.mbohlen.tools.jpresenter;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class PresentationHandler extends DefaultHandler
{
    private final int READ_DATA = 1;
    private final int READ_NOTHING = 0;

    private int state = READ_NOTHING;

    private Presentation presentation;

    /**
     * @see org.xml.sax.ContentHandler#characters(char[], int, int)
     */
    public void characters(char[] ch, int start, int length)
        throws SAXException
    {
        if (state == READ_DATA)
        {
            // System.out.print("characters: ");
            // System.out.println(ch);
        }
    }

    /**
     * @see org.xml.sax.ContentHandler#endElement(String, String, String)
     */
    public void endElement(String namespaceURI, String localName, String qName)
        throws SAXException
    {
        state = READ_NOTHING;
    }

    /**
     * @see org.xml.sax.ContentHandler#startElement(String, String, String, Attributes)
     */
    public void startElement(
        String namespaceURI,
        String localName,
        String qName,
        Attributes atts)
        throws SAXException
    {
        if (qName.equals("presentation"))
            presentation = new Presentation();

        if (qName.equals("slide"))
            addSlide(atts);
    }

    /**
     * Adds a new slide to the co
     * @param atts
     */
    private void addSlide(Attributes atts)
    {
        presentation.addSlide(
            new Slide(
                atts.getValue("title"),
                atts.getValue("image"),
                atts.getValue("audio")));
    }

    /**
     * Returns the parsed presentation.
     * @return Presentation the presentation.
     */
    public Presentation getPresentation()
    {
        return presentation;
    }
}
