package de.mbohlen.tools.jpresenter;

import java.awt.Image;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;

import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGImageDecoder;

public class Slide
{
    private String title;
    private String image;
    private String audio;

    public Slide(String title, String image, String audio)
    {
        this.title = title;
        this.image = image;
        this.audio = audio;
    }

    /**
     * Returns the audio.
     * @return String
     */
    public String getAudio()
    {
        return audio;
    }

    /**
     * Returns the image.
     * @return String
     */
    public String getImage()
    {
        return image;
    }

    /**
     * Returns the title.
     * @return String
     */
    public String getTitle()
    {
        return title;
    }

    public String toString()
    {
        return title;
    }

    /**
     * Loads the image of this slide and returns it.
     * @return Image the loaded image
     */
    public Image loadImage(File zipFile)
        throws FileNotFoundException, IOException
    {
        InputStream is = ZipUtils.loadEntryFromZip(zipFile, getImage());

        JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(is);
        Image result = decoder.decodeAsBufferedImage();

        is.close();
        return result;
    }

    public Clip loadSound(File zipFile)
        throws
            IOException,
            UnsupportedAudioFileException,
            LineUnavailableException
    {
        InputStream is = ZipUtils.loadEntryFromZip(zipFile, getAudio());

        AudioInputStream stream = AudioSystem.getAudioInputStream(is);
        is.close();

        AudioFormat format = stream.getFormat();

        /**
         * we can't yet open the device for ALAW/ULAW playback,
         * convert ALAW/ULAW to PCM
         */
        if ((format.getEncoding() == AudioFormat.Encoding.ULAW)
            || (format.getEncoding() == AudioFormat.Encoding.ALAW))
        {
            AudioFormat tmp =
                new AudioFormat(
                    AudioFormat.Encoding.PCM_SIGNED,
                    format.getSampleRate(),
                    format.getSampleSizeInBits() * 2,
                    format.getChannels(),
                    format.getFrameSize() * 2,
                    format.getFrameRate(),
                    true);
            stream = AudioSystem.getAudioInputStream(tmp, stream);
            format = tmp;
        }

        DataLine.Info info =
            new DataLine.Info(
                Clip.class,
                stream.getFormat(),
                ((int) stream.getFrameLength() * format.getFrameSize()));

        Clip clip = (Clip) AudioSystem.getLine(info);
        clip.open(stream);

        return clip;

    }
}
